from nttt import main, tidyup_translations

#tidyup_translations("C:\\Users\\Martin O'Hanlon-LT\\Documents\\temp\\coordinates\\", "C:\\Users\\Martin O'Hanlon-LT\\Documents\\temp\\coordinates\\fixed\\")
main()